@@ -120,7 +120,7 @@ static void      bookmarks_rename_button_cb (GtkWidget         *widget,
 static void      bookmarks_remove_button_cb (GtkWidget         *widget,
 					     GtkTreeView       *view);
 static void      bookmarks_ensure_valid     (void);
-static gboolean  bookmarks_read             (void);
+static gboolean  bookmarks_read             (const gchar       *filename);
 
 static gboolean  bookmarks_dup_finder       (GtkTreeModel   *model,
 					     GtkTreePath    *path,
@@ -169,7 +169,8 @@ static const GtkActionEntry popup_entries[] = {
 void
 yelp_bookmarks_init (void)
 {
-    gboolean read;
+    gboolean read = FALSE;
+    gchar   *filename = NULL;
 
     windows = NULL;
     actions_store = gtk_tree_store_new (4,
@@ -178,7 +179,13 @@ yelp_bookmarks_init (void)
 					G_TYPE_BOOLEAN,
 					G_TYPE_BOOLEAN);
 
-    read = bookmarks_read ();
+    filename = g_build_filename (g_get_home_dir (), ".gnome2",
+				 "yelp-bookmarks.xbel", NULL);
+
+    if (g_file_test (filename, G_FILE_TEST_EXISTS))
+	read = bookmarks_read (filename);
+
+    g_free (filename);
 }
 
 void
@@ -523,9 +530,12 @@ bookmarks_key_event_cb (GtkWidget   *widget,
 	switch (event->keyval) {
 	case GDK_BackSpace:
 	case GDK_Delete:
-	    gtk_tree_store_remove (actions_store, &iter);
-	    bookmarks_ensure_valid ();
-	    bookmarks_rebuild_menus ();
+	    if (!gtk_tree_model_iter_has_child (GTK_TREE_MODEL (actions_store),
+						&iter)) {
+		gtk_tree_store_remove (actions_store, &iter);
+		bookmarks_ensure_valid ();
+		bookmarks_rebuild_menus ();
+	    }
 	    break;
 	case GDK_Return:
 	case GDK_KP_Enter:
@@ -556,15 +566,17 @@ bookmarks_open_cb (GtkTreeView *view, GtkTreePath *path,
     YelpWindowData *data;
 
     gtk_tree_model_get_iter (GTK_TREE_MODEL (actions_store), &iter, path);
-    gtk_tree_model_get (GTK_TREE_MODEL (actions_store), &iter,
-			COL_NAME, &name,
-			COL_LABEL, &title, -1);
-
-    cur = windows;
-    data = cur->data;
-
-    g_signal_emit_by_name (data->window, "new_window_requested", name, NULL);
-
+    if (!gtk_tree_model_iter_has_child (GTK_TREE_MODEL (actions_store),
+					&iter)) {
+	gtk_tree_model_get (GTK_TREE_MODEL (actions_store), &iter,
+			    COL_NAME, &name,
+			    COL_LABEL, &title, -1);
+	
+	cur = windows;
+	data = cur->data;
+	
+	g_signal_emit_by_name (data->window, "new_window_requested", name, NULL);
+    }
 }
 
 
@@ -613,6 +625,9 @@ yelp_bookmarks_edit (void)
 
 	g_signal_connect (G_OBJECT (bookmarks_dialog), "response",
 			  G_CALLBACK (gtk_widget_hide), NULL);
+	g_signal_connect (G_OBJECT (bookmarks_dialog), "delete_event",
+			  G_CALLBACK (gtk_widget_hide_on_delete), NULL);
+	
 	renderer = gtk_cell_renderer_text_new ();
     
 	gtk_tree_view_insert_column_with_attributes
@@ -681,7 +696,10 @@ bookmarks_configure_cb (GtkWidget *widget, GdkEventConfigure *event,
 static void
 selection_changed_cb (GtkTreeSelection *selection, gpointer data)
 {
-    if (gtk_tree_selection_get_selected (selection, NULL, NULL)) {
+    GtkTreeIter iter;
+    if (gtk_tree_selection_get_selected (selection, NULL, &iter) &&
+	!gtk_tree_model_iter_has_child (GTK_TREE_MODEL (actions_store), 
+					&iter)) {
 	/*A row is highlighted - sensitise the various widgets*/
 	gtk_widget_set_sensitive (GTK_WIDGET (edit_open_button), TRUE);
 	gtk_widget_set_sensitive (GTK_WIDGET (edit_rename_button), TRUE);
@@ -914,11 +932,12 @@ yelp_bookmarks_write (void)
 				    COL_LABEL, &label,
 				    -1);
 		xmlTextWriterStartElement (file, BAD_CAST "bookmark");
- 		xmlTextWriterWriteAttribute (file, "href", name);
+ 		xmlTextWriterWriteAttribute (file, BAD_CAST "href", 
+					     BAD_CAST name);
 
 		rc = xmlTextWriterWriteElement (file,
 						BAD_CAST "title",
-						label);
+						BAD_CAST label);
 
 		rc = xmlTextWriterStartElement (file, BAD_CAST "info");
 		rc = xmlTextWriterStartElement (file, BAD_CAST "metadata");
@@ -947,7 +966,7 @@ yelp_bookmarks_write (void)
 }
 
 static gboolean
-bookmarks_read (void)
+bookmarks_read (const gchar *filename)
 {
     xmlParserCtxtPtr   parser;
     xmlXPathContextPtr xpath;
@@ -955,12 +974,8 @@ bookmarks_read (void)
     xmlDocPtr          doc;
 
     gboolean  ret = TRUE;
-    gchar    *filename;
     gint      i;
 
-    filename = g_build_filename (g_get_home_dir (), ".gnome2", 
-				 "yelp-bookmarks.xbel", NULL);
-
     parser = xmlNewParserCtxt ();
     doc = xmlCtxtReadFile (parser, filename, NULL,
 			   XML_PARSE_NOBLANKS | XML_PARSE_NOCDATA  |
@@ -968,33 +983,34 @@ bookmarks_read (void)
 			   XML_PARSE_NONET    );
 
     if (doc == NULL) {
-	g_free (filename);
 	if (parser)
 		xmlFreeParserCtxt (parser);
 	return FALSE;
     }
 
     xpath = xmlXPathNewContext (doc);
-    obj = xmlXPathEvalExpression ("/xbel/bookmark", xpath);
+    obj = xmlXPathEvalExpression (BAD_CAST "/xbel/bookmark", xpath);
     for (i = 0; i < obj->nodesetval->nodeNr; i++) {
 	xmlNodePtr node = obj->nodesetval->nodeTab[i];
 	gchar *uri;
-	uri = xmlGetProp (node, "href");
+	uri = (gchar *) xmlGetProp (node, BAD_CAST "href");
 	if (uri) {
 	    xmlXPathObjectPtr title_obj;
 	    xpath->node = node;
-	    title_obj = xmlXPathEvalExpression ("string(title[1])", xpath);
+	    title_obj = xmlXPathEvalExpression (BAD_CAST "string(title[1])", 
+						xpath);
 	    xpath->node = NULL;
 
 	    if (title_obj->stringval)
-		bookmarks_add_bookmark (uri, title_obj->stringval, FALSE);
+		bookmarks_add_bookmark ((const gchar *) uri, 
+					(const gchar *) title_obj->stringval, 
+					FALSE);
 
 	    xmlXPathFreeObject (title_obj);
 	    xmlFree (uri);
 	}
     }
     
-    g_free (filename);
     if (obj)
 	xmlXPathFreeObject (obj);
     if (xpath)
