@@ -28,6 +28,7 @@
 
 #include <glib/gi18n.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
+#include <gdk/gdkkeysyms.h>
 #include <gtk/gtk.h>
 #include <bonobo/bonobo-main.h>
 #include <libgnomevfs/gnome-vfs.h>
@@ -54,6 +55,10 @@
 #ifdef ENABLE_INFO
 #include "yelp-info-pager.h"
 #endif
+#ifdef ENABLE_SEARCH
+#include "yelp-search-pager.h"
+#include "gtkentryaction.h"
+#endif
 
 #ifdef YELP_DEBUG
 #define d(x) x
@@ -174,6 +179,7 @@ static void    window_about_cb          (GtkAction *action, YelpWindow *window);
 static void    window_copy_link_cb      (GtkAction *action, YelpWindow *window);
 static void    window_open_link_cb      (GtkAction *action, YelpWindow *window);
 static void    window_open_link_new_cb  (GtkAction *action, YelpWindow *window);
+static void    window_copy_mail_cb      (GtkAction *action, YelpWindow *window);
 
 static gboolean           window_load_async      (YelpLoadData *data);
 
@@ -195,12 +201,18 @@ static void               location_response_cb    (GtkDialog       *dialog,
 static gboolean    window_find_action             (YelpWindow        *window);
 static void        window_find_entry_changed_cb   (GtkEditable       *editable,
 						   gpointer           data);
+static void        window_find_entry_activate_cb   (GtkEditable       *editable,
+						    gpointer         data);
+static gboolean    window_find_entry_key_pressed_cb (GtkWidget       *widget,
+						     GdkEventKey     *key,
+						     gpointer         data);
 static void        window_find_save_settings      (YelpWindow        *window);
 static void        window_find_buttons_set_sensitive (YelpWindow      *window,
 						      gboolean        sensitive);
 static void        window_find_clicked_cb         (GtkWidget         *button,
 						   YelpWindow        *window);
-
+static void        window_find_next_cb            (GtkAction *action, 
+						   YelpWindow *window);
 static gboolean    tree_model_iter_following      (GtkTreeModel      *model,
 						   GtkTreeIter       *iter);
 
@@ -235,6 +247,8 @@ struct _YelpWindowPriv {
 
     /* Popup menu*/
     GtkWidget      *popup;
+    gint            merge_id;
+    GtkWidget      *maillink;
     gchar          *uri;
 
     /* Location Information */
@@ -327,6 +341,11 @@ static const GtkActionEntry entries[] = {
       "<Control>F",
       NULL,
       G_CALLBACK (window_find_cb) },
+    { "FindNext", NULL,
+      N_("Find Ne_xt"),
+      "<Control>G",
+      NULL,
+      G_CALLBACK (window_find_next_cb) },
     { "Preferences", GTK_STOCK_PREFERENCES,
       N_("_Preferences"),
       NULL, NULL,
@@ -339,7 +358,7 @@ static const GtkActionEntry entries[] = {
       G_CALLBACK (window_reload_cb) },
 
     { "TextCursor", NULL,
-      NULL,
+      "TextCursor",
       "F7",
       NULL,
       G_CALLBACK (window_enable_cursor_cb) },
@@ -406,7 +425,12 @@ static const GtkActionEntry entries[] = {
       N_("_About"),
       NULL,
       NULL,
-      G_CALLBACK (window_about_cb) }
+      G_CALLBACK (window_about_cb) },
+    { "CopyMail", NULL,
+      N_("Copy _Email Address"),
+      NULL,
+      NULL,
+      G_CALLBACK (window_copy_mail_cb) }
 };
 
 GType
@@ -812,6 +836,15 @@ window_do_load (YelpWindow  *window,
     case YELP_DOC_TYPE_TOC:
 	handled = window_do_load_pager (window, doc_info, frag_id);
 	break;
+    case YELP_DOC_TYPE_SEARCH:
+#ifdef ENABLE_SEARCH
+	handled = window_do_load_pager (window, doc_info, frag_id);
+	break;
+#else
+	g_set_error (&error, YELP_ERROR, YELP_ERROR_FORMAT,
+		     _("Search is not supported in this version."));
+	break;
+#endif
     case YELP_DOC_TYPE_DOCBOOK_SGML:
 	g_set_error (&error, YELP_ERROR, YELP_ERROR_FORMAT,
 		     _("SGML documents are no longer supported. Please ask "
@@ -891,6 +924,31 @@ window_error (YelpWindow *window, GError *error, gboolean pop)
     gtk_widget_destroy (dialog);
 }
 
+#ifdef ENABLE_SEARCH
+static char *
+encode_search_uri (const char *search_terms)
+{
+    return g_strdup_printf ("x-yelp-search:%s", search_terms);
+}
+
+static void
+search_activated (GtkAction *action,
+                 YelpWindow *window)
+{
+    const char *search_terms;
+    char *uri;
+
+    search_terms = gtk_entry_action_get_text (GTK_ENTRY_ACTION (action));
+
+    uri = encode_search_uri (search_terms);
+
+    yelp_window_load (window, uri);
+
+    g_free (uri);
+}
+#endif
+
+
 static void
 window_populate (YelpWindow *window)
 {
@@ -912,6 +970,17 @@ window_populate (YelpWindow *window)
     gtk_action_group_add_actions (priv->action_group,
 				  entries, G_N_ELEMENTS (entries),
 				  window);
+    
+#ifdef ENABLE_SEARCH
+    action =  gtk_entry_action_new ("Search",
+				    _("Search"),
+				    _("Search for other documentation"),
+				    NULL);
+    g_signal_connect (G_OBJECT (action), "activate",
+		      G_CALLBACK (search_activated), window);
+#endif
+    gtk_action_group_add_action (priv->action_group, action);
+
     priv->ui_manager = gtk_ui_manager_new ();
     gtk_ui_manager_insert_action_group (priv->ui_manager, priv->action_group, 0);
 
@@ -928,6 +997,14 @@ window_populate (YelpWindow *window)
 	window_error (window, error, FALSE);
     }
 
+#ifdef ENABLE_SEARCH
+    if (!gtk_ui_manager_add_ui_from_file (priv->ui_manager,
+					  DATADIR "/yelp/ui/yelp-search-ui.xml",
+					  &error)) {
+	window_error (window, error, FALSE);
+    }
+#endif
+
     yelp_bookmarks_register (window);
 
     gtk_ui_manager_ensure_update (priv->ui_manager);
@@ -938,7 +1015,9 @@ window_populate (YelpWindow *window)
     action = gtk_action_group_get_action (priv->action_group, "GoForward");
     if (action)
 	g_object_set (G_OBJECT (action), "sensitive", FALSE, NULL);
-    priv->popup = gtk_ui_manager_get_widget(priv->ui_manager, "ui/popup");
+    priv->popup = gtk_ui_manager_get_widget(priv->ui_manager, "ui/main_popup");
+    priv->maillink = gtk_ui_manager_get_widget(priv->ui_manager, "ui/mail_popup");
+    priv->merge_id = gtk_ui_manager_new_merge_id (priv->ui_manager);
 
     priv->pane = gtk_hpaned_new ();
     gtk_widget_ref (priv->pane);
@@ -1039,6 +1118,10 @@ window_populate_find (YelpWindow *window, GtkWidget *find_bar)
     priv->find_entry = gtk_entry_new ();
     g_signal_connect (G_OBJECT (priv->find_entry), "changed",
 		      G_CALLBACK (window_find_entry_changed_cb), window);
+    g_signal_connect (G_OBJECT (priv->find_entry), "activate",
+		      G_CALLBACK (window_find_entry_activate_cb), window);
+    g_signal_connect (G_OBJECT (priv->find_entry), "key-press-event",
+		      G_CALLBACK (window_find_entry_key_pressed_cb), window);
     gtk_box_pack_start (GTK_BOX (box), priv->find_entry, TRUE, TRUE, 0);
 
     item = gtk_tool_item_new ();
@@ -1156,6 +1239,15 @@ window_do_load_pager (YelpWindow  *window,
 	case YELP_DOC_TYPE_TOC:
 	    pager = YELP_PAGER (yelp_toc_pager_get ());
 	    break;
+	case YELP_DOC_TYPE_SEARCH:
+#ifdef ENABLE_SEARCH
+	    pager = YELP_PAGER (yelp_search_pager_get (doc_info));
+	    break;
+#else
+	    g_set_error (&error, YELP_ERROR, YELP_ERROR_FORMAT,
+			 _("Search is not supported in this version."));
+	    break;
+#endif
 	default:
 	    break;
 	}
@@ -1486,7 +1578,7 @@ window_handle_page (YelpWindow   *window,
     context->length = strlen (page->contents);
 
     uri = yelp_doc_info_get_uri (priv->current_doc,
-				 page->page_id,
+				 priv->current_frag,
 				 YELP_URI_TYPE_FILE);
 
     d (g_print ("  uri            = %s\n", uri));
@@ -1730,6 +1822,20 @@ html_popupmenu_requested_cb (YelpHtml *html,
 {
     YelpWindow *window = YELP_WINDOW (user_data);
 
+    gtk_ui_manager_remove_ui (window->priv->ui_manager,
+			      window->priv->merge_id);
+
+
+    if (g_str_has_prefix(uri, "mailto:")) {
+	gtk_ui_manager_add_ui (window->priv->ui_manager,
+			       window->priv->merge_id,
+			       "/ui/main_popup",
+			       "CpMail",
+			       "CopyMail",
+			       GTK_UI_MANAGER_MENUITEM,
+			       FALSE);
+
+    }
     window->priv->uri = g_strdup (uri);
     gtk_menu_popup (GTK_MENU (window->priv->popup),
 		    NULL, NULL, NULL, NULL, 3, gtk_get_current_event_time());
@@ -1773,7 +1879,7 @@ tree_drag_data_get_cb (GtkWidget         *widget,
     GtkTreeSelection *tree_selection;
     GtkTreeModel     *model;
     GtkTreeIter       iter;
-    gchar *id, *uri;
+    gchar *id, *uri = NULL;
 
     g_return_if_fail (YELP_IS_WINDOW (window));
 
@@ -1800,7 +1906,7 @@ tree_drag_data_get_cb (GtkWidget         *widget,
     case TARGET_URI_LIST:
 	gtk_selection_data_set (selection,
 				selection->target,
-				8, uri, strlen (uri));
+				8, (const guchar *) uri, strlen (uri));
 	break;
     default:
 	g_assert_not_reached ();
@@ -2186,6 +2292,25 @@ window_open_link_new_cb (GtkAction *action, YelpWindow *window)
     g_free (window->priv->uri);
 };
 
+static void
+window_copy_mail_cb (GtkAction *action, YelpWindow *window)
+{
+    /*Do things the simple way.
+     * Split the string and remove any query bit then
+     * remove the first 7 chars as they should be mailto:
+     */
+    gchar **split_string = g_strsplit (window->priv->uri, "?", 2);
+    
+    gchar *mail_address = &split_string[0][7];
+    
+    gtk_clipboard_set_text (gtk_clipboard_get (gdk_atom_intern ("CLIPBOARD", 
+								TRUE)),
+			    mail_address,
+			    -1);
+    g_free (window->priv->uri);
+    g_strfreev(split_string);
+}
+
 static void
 window_about_cb (GtkAction *action, YelpWindow *window)
 {
@@ -2279,6 +2404,20 @@ window_find_again (YelpWindow *window, gboolean forward)
     return yelp_html_find_again (priv->html_view, forward);
 }
 
+static void
+window_find_next_cb (GtkAction *action, YelpWindow *window)
+{
+    YelpWindowPriv * priv;
+
+    priv = window->priv;
+    if (!window_find_again (window, TRUE)) {
+	gtk_widget_set_sensitive (GTK_WIDGET (priv->find_next), FALSE);
+	gtk_widget_set_sensitive (GTK_WIDGET (priv->find_prev), TRUE);
+    } else {
+	window_find_buttons_set_sensitive (window, TRUE);
+    }
+}
+
 static void
 window_find_clicked_cb (GtkWidget  *widget,
 			YelpWindow *window)
@@ -2341,7 +2480,54 @@ window_find_entry_changed_cb (GtkEditable *editable,
 }
 
 static void
-window_find_buttons_set_sensitive (YelpWindow *window,
+window_find_entry_activate_cb (GtkEditable *editable,
+				gpointer data)
+{
+    YelpWindow *window;
+    YelpWindowPriv *priv;
+
+    g_return_if_fail (YELP_IS_WINDOW (data));
+
+    window = YELP_WINDOW (data);
+    priv = window->priv;
+
+    window_find_buttons_set_sensitive (window, FALSE);
+
+    /* search forward and update the buttons */
+    if (!window_find_again (window, TRUE)) {
+	gtk_widget_set_sensitive (GTK_WIDGET (priv->find_next), FALSE);
+	gtk_widget_set_sensitive (GTK_WIDGET (priv->find_prev), TRUE);
+    } else {
+	window_find_buttons_set_sensitive (window, TRUE);
+    }
+}
+
+static gboolean
+window_find_entry_key_pressed_cb (GtkWidget    *widget,
+				  GdkEventKey  *key,
+				  gpointer     data)
+{
+    YelpWindow     *window;
+    YelpWindowPriv *priv;
+
+    if (key->keyval == GDK_Escape) {
+	g_return_val_if_fail (YELP_IS_WINDOW (data), FALSE);
+
+	window = YELP_WINDOW (data);
+	priv = window->priv;
+
+	d (g_print ("window_find_entry_key_pressed_cb\n"));
+
+	gtk_widget_hide_all (priv->find_bar);
+
+	return TRUE;
+    }
+
+    return FALSE;
+}
+
+static void
+window_find_buttons_set_sensitive (YelpWindow  *window,
 				   gboolean    sensitive)
 {
     YelpWindowPriv *priv;
